@@ -190,6 +190,7 @@ class RoborockMopModeS8ProUltra(RoborockMopModeCode):
     deep = 301
     deep_plus = 303
     fast = 304
+    custom = 302
 
 
 class RoborockMopIntensityCode(RoborockEnum):
@@ -223,6 +224,8 @@ class RoborockDockErrorCode(RoborockEnum):
     duct_blockage = 34
     water_empty = 38
     waste_water_tank_full = 39
+    dirty_tank_latch_open = 44
+    no_dustbin = 46
 
 
 class RoborockDockTypeCode(RoborockEnum):
@@ -231,7 +234,9 @@ class RoborockDockTypeCode(RoborockEnum):
     auto_empty_dock = 1
     empty_wash_fill_dock = 3
     auto_empty_dock_pure = 5
+    s7_max_ultra_dock = 6
     s8_dock = 7
+    q_revo = 8
 
 
 class RoborockDockDustCollectionModeCode(RoborockEnum):
